import numpy as np
import sys, os
import glob
from lightningcast import utils
import pickle
from lightningcast import solar_angles
from pyproj import Proj

goes = "goes_east"
datadir = f"/ships19/grain/jcintineo/GLM/{goes}/CONUS/2020/"

dates = np.sort(glob.glob(f"{datadir}/2*"))

stride = 4

for date in dates:
    print(date)
    preds_labs_file = f"{date}/all_preds_labs.pkl"
    try:
        preds_labs = pickle.load(open(preds_labs_file, "rb"))
    except FileNotFoundError:
        print("FileNotFound")
        continue

    samplelist = preds_labs["samplelist"]
    if not (os.path.isfile(samplelist)):  # fix samplelist, if necessary
        parts = samplelist.split("offshore_lists")
        samplelist = f"{parts[0]}/offshore_lists/{goes}/{parts[1]}"
        preds_labs["samplelst"] = samplelist

    # samples should have same order and length of preds and labs dim0
    samples = pickle.load(open(samplelist, "rb"))
    all_solzen = np.empty(preds_labs["preds"].shape)
    for ii, samp in enumerate(samples):
        # make solzen
        dt = samp["abidt"]

        try:  # we're assuming the lats and lons are the same for each file
            irlons
        except NameError:
            datasets = {
                "x": {"atts": ["ALL"]},
                "y": {"atts": ["ALL"]},
                "goes_imager_projection": {"atts": ["ALL"]},
            }
            datasets = utils.read_netcdf(samp["CH13"], datasets)
            x = datasets["x"]["data"][samp["startX"] : samp["endX"] : stride]
            y = datasets["y"]["data"][samp["startY"] : samp["endY"] : stride]
            satheight = datasets["goes_imager_projection"]["atts"][
                "perspective_point_height"
            ]
            lon0 = datasets["goes_imager_projection"]["atts"][
                "longitude_of_projection_origin"
            ]
            sweep = datasets["goes_imager_projection"]["atts"]["sweep_angle_axis"]
            projection = Proj(proj="geos", h=satheight, lon_0=lon0, sweep=sweep)
            xx, yy = np.meshgrid(x, y)
            irlons, irlats = projection(xx, yy, inverse=True)
        solzen, _ = solar_angles.solar_angles(dt=dt, lat=irlats, lon=irlons)
        all_solzen[ii] = solzen

    # now save new pickle
    preds_labs["solzen"] = all_solzen
    pickle.dump(preds_labs, open(preds_labs_file, "wb"))
#
