from netCDF4 import Dataset
import xarray as xr
import numpy as np
import os, sys
import glob
import time

t0 = time.time()

datadir = os.path.join("/data", "jcintineo", "AI", "OCONUS")
outfile = os.path.join(datadir, "merged.nc")
ncs = np.sort(glob.glob(os.path.join(datadir, "201805*.nc")))
chunksize = 300

create = False
print("create new netcdf =", create)
time.sleep(5)

for ii in range(len(ncs)):
    print(ncs[ii])
    nc = Dataset(ncs[ii], "r")
    if ii == 0 and create:
        out_nc = Dataset(outfile, "w")
        # set dims
        for d in nc.dimensions:
            if d == "t":
                out_nc.createDimension("t", None)
            else:
                out_nc.createDimension(d, nc.dimensions[d].size)
        # set and fill vars
        for v in nc.variables:
            # chunking
            chunksizes = []
            for d in nc.variables[v].dimensions:
                if d == "t":
                    chunksizes.append(chunksize)
                else:
                    chunksizes.append(nc.dimensions[d].size)

            var = out_nc.createVariable(
                v,
                nc.variables[v].dtype,
                nc.variables[v].dimensions,
                chunksizes=chunksizes,
            )
            var[:] = nc.variables[v][:]
        # set global atts
        for att in nc.ncattrs():
            out_nc.setncattr(att, nc.getncattr(att))

    else:
        out_nc = Dataset(outfile, "a")
        curr_ntimes = out_nc.variables["CH13"].shape[0]
        new_ntimes = nc.variables["CH13"].shape[0]
        for v in out_nc.variables:
            out_nc.variables[v][
                curr_ntimes : (curr_ntimes + new_ntimes)
            ] = nc.variables[v][:]

    # close netcdfs
    out_nc.close()
    nc.close()

print(time.time() - t0)
# sys.exit()
