import numpy as np
import netCDF4
import glob
import os, sys
from datetime import datetime, timedelta
import inspect

current_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
sys.path.insert(0, os.path.dirname(current_dir))
from lightningcast import utils

dataroot = "/data/PLTG/"
product_dir = f"{dataroot}/products/"
log_dir = f"{dataroot}/logs/"

# First get list of checked files
checked_filename = f"{log_dir}/checked_netcdf.txt"
with open(checked_filename) as file:
    files = [line.rstrip() for line in file]

# If this file no longer exists, don't include it in checked_files list
checked_files = []  # we write this out at the end
for file in files:
    if os.path.isfile(file):
        checked_files.append(file)

bad_files = []

for root, directories, files in os.walk(product_dir):
    for subdirectory in directories:
        full_dir = f"{root}/{subdirectory}"
        if full_dir.endswith("netcdf"):
            files = glob.glob(f"{full_dir}/*nc")
            if len(files) > 0:
                for f in files:
                    if (
                        f not in checked_files
                    ):  # if not already checked, check for bad DQFs
                        nc = netCDF4.Dataset(f, "r")
                        dqfs = nc.variables["DQF"][:]
                        nc.close()
                        if np.max(dqfs) > 0:
                            # log it
                            bad_files.append(f)
                        checked_files.append(f)

# Write out checked files
if len(checked_files) > 0:
    with open(checked_filename, "w") as file:
        for fname in checked_files:
            file.write(fname + "\n")

# Notify about bad files
if len(bad_files) > 0:
    to = "john.cintineo@ssec.wisc.edu"
    subject = "LightningCast bad DQFs found"

    message = os.uname()[1] + "\n"
    message += ("\n").join(bad_files)

    utils.send_email(subject, message, to)
