import glob
import os, sys
import netCDF4
import argparse
import pickle
from datetime import datetime, timedelta
from lightningcast import utils
import numpy as np
import predict_ltg as pl


def create_lustre_samplelist(
    startdt,
    enddt,
    abipattern,
    ltgpattern,
    outdir,
    outfilename=None,
    samplelist=None,
    threshold=0.025,
):

    if samplelist is None:
        samplelist = []
    elif isinstance(samplelist, str):  # assume we need to load it
        samplelist = pickle.load(open(samplelist, "rb"))

    # Each tuple contains (startY,endY,startX,endX), in 2-km ABI CONUS FGF data space.
    # These values were chosen to make 6 equal patches with shape=(736,832). Each
    # dimension is divisible by 32, which is more optimal for training CNNs.
    # The 4 western-most columns and 28 northern-most rows are excluded to get the
    # dimensions right. We chose to use the eastern and southern ABI grid edges
    # since there is more good data there than the northern and western edges, which
    # contain space looks.
    imgpt_list = [
        (28, 764, 4, 836),  # (28,764,4,836),  (28,764,373,1205) = No spacelook
        (28, 764, 836, 1668),  # (28,764,836,1668), (28,764,1205,2037) = No spacelook
        (28, 764, 1668, 2500),
        (764, 1500, 4, 836),
        (764, 1500, 836, 1668),
        (764, 1500, 1668, 2500),
    ]

    # large GOES-West sector
    # imgpt_list = [(400,2900,1100,4100)]
    # timgpt_list = [(400,2900,1100,4100)]

    # PACUS sector
    imgpt_list = timgpt_list = [(422, 422 + 1500, 1462, 1462 + 2500)]

    #  imgpt_list = timgpt_list = [(0,1500,0,2500)]
    # for "OCONUS" GOES-East sector
    # timgpt_list = [(500,3000,700,3900)] #[(0,2500,0,3200)] #for truth data /700,3900,500,3000
    # imgpt_list = [(500,3000,700,3900)] #for imager data

    assert startdt < enddt

    dt = startdt
    while startdt <= dt <= enddt:
        # add 1 hour, b/c ltg files' timestamps are the END time of the accumulation.
        ltgdt = dt + timedelta(minutes=60)
        ltg_file_patt = ltgdt.strftime(ltgpattern)
        try:
            ltg_file = glob.glob(ltg_file_patt)[0]
        except IndexError:
            # increment dt by 1 minute
            dt = dt + timedelta(minutes=1)
        else:
            abifiles = np.sort(glob.glob(dt.strftime(abipattern)))
            # match and save the abi files
            if len(abifiles) >= 16:  # make sure all channels are present
                abidict = {}
                for abif in abifiles:
                    basefile = os.path.basename(abif)
                    ch = "CH" + basefile.split("_")[1].split("-")[-1].split("C")[-1]
                    abidict[ch] = abif

                # nc = netCDF4.Dataset(ltg_file,'r')
                for ii, ipts in enumerate(imgpt_list):
                    sampledict = {}
                    # Read the ltg_file and test to make sure there is some good ltg data in the patch defined by ipts.
                    # We do this because each sample (actually, batch) should have all classes that we are
                    # interested in predicting.
                    startY = ipts[0]
                    endY = ipts[1]
                    startX = ipts[2]
                    endX = ipts[3]
                    try:
                        tstartY = timgpt_list[ii][0]
                        tendY = timgpt_list[ii][1]
                        tstartX = timgpt_list[ii][2]
                        tendX = timgpt_list[ii][3]
                    except NameError:
                        tstartY = ipts[0]
                        tendY = ipts[1]
                        tstartX = ipts[2]
                        tendX = ipts[3]

                    # see if there is any missing data
                    # cont = False
                    # for abif in abifiles:
                    #  basefile = os.path.basename(abif)
                    #  ch = 'CH' + basefile.split('_')[1].split('-')[-1].split('C')[-1]
                    #  if(ch == 'CH04' or ch == 'CH06'): continue #not sure why, but CH04 seems to always have some masked pixels
                    #  abi_data = pl.get_abi_data(dt,ch[2:],'/arcdata/goes/grb/goes16/','RadC',startY=startY,endY=endY,startX=startX,endX=endX)
                    #  if(np.ma.is_masked(abi_data)): #if any pts masked in this patch in any channel, skip
                    #    print('here',abif,ipts,abi_data.shape)

                    #    cont=True; break
                    # if(cont): continue #move on to next ipts

                    # assuming variable name is in the path; the immediate directory containing the files.
                    #   varname = os.path.basename(os.path.dirname(ltg_file))

                    # check if we're using 8-km data
                    #   if(nc.spatial_resolution == "reduced from 2-km to 8-km"):
                    #     ltgdatapatch = nc.variables[varname][startY//4:endY//4,startX//4:endX//4]
                    #   else: #else assume 2-km target data
                    #     ltgdatapatch = nc.variables[varname][startY:endY,startX:endX]

                    # if(np.max(ltgdatapatch) >= threshold): #good target data, add the patch as a sample
                    #   ltgfrac = np.count_nonzero(ltgdatapatch > 0.1) / ((endY-startY) * (endX-startX))
                    if True:  # ltgfrac >= threshold):
                        sampledict["startY"] = startY
                        sampledict["endY"] = endY
                        sampledict["startX"] = startX
                        sampledict["endX"] = endX
                        sampledict["tstartY"] = tstartY
                        sampledict["tendY"] = tendY
                        sampledict["tstartX"] = tstartX
                        sampledict["tendX"] = tendX
                        sampledict["abidt"] = dt
                        sampledict["target_file"] = ltg_file
                        for ch, abif in abidict.items():
                            sampledict[ch] = abif
                        samplelist.append(sampledict)
                        print("Added", dt, startY, endY, startX, endX, "patch")

            # nc.close() #close the ltg_file

            else:
                print(dt, ": Only", len(abifiles), "present")

            # increment dt by 1 minute
            dt = dt + timedelta(minutes=1)

    # save the samplelist
    start_ts = startdt.strftime("%Y%m%d")
    end_ts = enddt.strftime("%Y%m%d")
    now = datetime.utcnow().strftime("%Y%m%d-%H%M")
    utils.mkdir_p(outdir)
    if outfilename:
        outfile = os.path.join(outdir, outfilename)
    else:
        outfile = f"{outdir}/samplelist_s{start_ts}_e{end_ts}_c{now}.pkl"
    pickle.dump(samplelist, open(outfile, "wb"))

    print("saved " + outfile)


if __name__ == "__main__":

    parser = argparse.ArgumentParser(
        description="This script uses target/truth data in --ltgpattern with satellite data \
                                                in --abipattern to create a samplelist; i.e., a list of dictionaries \
                                                used to feed into a generator for training deep-learning models from \
                                                data on lustre. You probably want to create a samplelist for both the \
                                                training and the validation sets/generators. Note that the patch endpoints\
                                                may be hardcoded in the function, so be sure those are correct."
    )
    parser.add_argument(
        "start_time",
        help="datetime to begin searching for ABI times. E.g., 20201001-2021",
        nargs=1,
        type=str,
    )
    parser.add_argument(
        "end_time",
        help="datetime to end search for ABI times. E.g., 20201011-0342",
        nargs=1,
        type=str,
    )
    parser.add_argument(
        "-a",
        "--abipattern",
        help="ABI dir to search for patterns to match. \
                       Default='/arcdata/goes/grb/goes16/Y/Y_m_d_j/abi/L1b/RadC/*_sYjHM*'",
        default=["/arcdata/goes/grb/goes16/%Y/%Y_%m_%d_%j/abi/L1b/RadC/*_s%Y%j%H%M*"],
        nargs=1,
        type=str,
    )
    parser.add_argument(
        "-l",
        "--ltgpattern",
        help="GLM dir to search for patterns to match. \
                       Default='/apollo/grain/saved_probsevere_data/lightning/Ymd/GLM/goes_east/FED_accum_60min_2km/Ymd-HM*'",
        default=[
            "/apollo/grain/saved_probsevere_data/lightning/%Y%m%d/GLM/goes_east/FED_accum_60min_2km/%Y%m%d-%H%M*"
        ],
        nargs=1,
        type=str,
    )
    parser.add_argument(
        "-s",
        "--samplelist_pickle",
        help="Append to this samplelist pickle file.",
        nargs=1,
        default=[None],
        type=str,
    )
    parser.add_argument(
        "-t",
        "--threshold",
        help="Threshold fraction for GLM FED to achieve before adding a sample patch. \
                                               Default=0.025.",
        default=[0.025],
        nargs=1,
        type=float,
    )
    parser.add_argument(
        "-o",
        "--outdir",
        help="Output directory. Default=${PWD}/OUTPUT/",
        nargs=1,
        default=[os.environ["PWD"] + "/OUTPUT/"],
        type=str,
    )
    parser.add_argument(
        "-on",
        "--outname",
        help="Outfile name. Default is 'samplelist_s{start_time}_e{end_time}_c{creation_time}.pkl'.",
        default=[None],
        nargs=1,
        type=str,
    )

    args = parser.parse_args()

    startdt = datetime.strptime(args.start_time[0], "%Y%m%d-%H%M")
    enddt = datetime.strptime(args.end_time[0], "%Y%m%d-%H%M")

    if args.samplelist_pickle[0] is not None:
        samplelist = pickle.load(open(args.samplelist_pickle[0], "rb"))
    else:
        samplelist = None

    create_lustre_samplelist(
        startdt,
        enddt,
        args.abipattern[0],
        args.ltgpattern[0],
        args.outdir[0],
        samplelist=samplelist,
        outfilename=args.outname[0],
        threshold=args.threshold[0],
    )
