import numpy as np
import pandas as pd
import os, sys
from multiprocessing import Pool

# FIXME: try to use mulitprocessing to speed this up


def make_recs(df):

    records = []

    num_seq = (
        7 - 1
    )  # including endpoint times. E.g., 30-min sequence would require 7 frames.
    # subtract 1 because of 0-based indexing.
    deltaT = num_seq * 5  # multiply by 5 if using ABI CONUS grids
    grid_vars = ["ch13"]

    scalar_vars = [
        "lat",
        "lon",
        "ABIx",
        "ABIy",
        "ABI_time",
        "maxFED_t0",
        "maxFED_t5",
        "maxFED_t10",
        "maxFED_t15",
        "maxFED_t20",
        "maxFED_t30",
        "ltg5to30",
        "sst",
        "mlcape",
        "u925",
        "v925",
        "u850",
        "v850",
        "u700",
        "v700",
        "u500",
        "v500",
    ]

    for ind in inds:
        print(ind)
        x = ind[0]
        y = ind[1]

        rec = []
        subdf = df[(df.ABIx == x) & (df.ABIy == y)]
        subdf = subdf.sort_values(["ABI_time"], ascending=[True])

        times = np.array(subdf.ABI_time)
        timediffs = (times[num_seq:] - times[0:-(num_seq)]).astype("timedelta64[m]")

        # padding front end with zeros
        timediffs = np.concatenate((np.zeros(num_seq), timediffs.astype(int)))
        good_timeind = np.where(timediffs == deltaT)

        for gti in good_timeind[0]:
            rec.append(np.stack(subdf.ch13[gti - num_seq : gti + 1]))
            for sv in scalar_vars:
                rec.append(subdf.iloc[gti][sv])
        # add to records if not empty
        if rec:
            (records.append(rec))

    return records


if __name__ == "__main__":

    # nthreads = 12
    # pool = Pool(nthreads)

    df = pd.read_pickle("ch13.pkl")
    X = pd.unique(df.ABIx)
    Y = pd.unique(df.ABIy)

    # df = df.sort_values(['ABIx','ABIy','ABI_time'], ascending=[True,True,True]).reset_index(drop=True)

    # subdf = df[(df.ABIx == X[0]) & (df.ABIy == Y[0])]

    # print(subdf)
    # print(len(subdf))
    # print(subdf.index[-1])
    # sys.exit()

    # records = []

    # options = {}
    # options['num_seq'] = num_seq
    # options['deltaT'] = deltaT
    # options['grid_vars'] = grid_vars
    # options['scalar_vars'] = scalar_vars

    # import itertools

    # inds = []
    # for x in X:
    #  inds.append([ii for ii in zip(itertools.repeat(x),Y)])
    # concatenate lists in results
    # allinds = list(itertools.chain.from_iterable(inds))

    # list_of_lists = []
    # interval = int(np.ceil(len(allinds)/float(nthreads)))
    # for n in range(nthreads):
    #  list_of_lists.append(allinds[n*interval:n*interval+interval+1])

    # records = pool.starmap(make_recs,zip(list_of_lists,itertools.repeat(df),itertools.repeat(options)))
    # pool.close()
    # pool.join()
    ##############################################

    records = []

    num_seq = (
        7 - 1
    )  # including endpoint times. E.g., 30-min sequence would require 7 frames.
    # subtract 1 because of 0-based indexing.
    deltaT = num_seq * 5  # multiply by 5 if using ABI CONUS grids
    grid_vars = ["ch13"]

    scalar_vars = [
        "lat",
        "lon",
        "ABIx",
        "ABIy",
        "ABI_time",
        "maxFED_t0",
        "maxFED_t5",
        "maxFED_t10",
        "maxFED_t15",
        "maxFED_t20",
        "maxFED_t30",
        "ltg5to30",
        "sst",
        "mlcape",
        "u925",
        "v925",
        "u850",
        "v850",
        "u700",
        "v700",
        "u500",
        "v500",
    ]

    for x in X:
        if x > 0:
            print(x)
            for y in Y:
                subdf = df[(df.ABIx == x) & (df.ABIy == y)]
                subdf = subdf.sort_values(["ABI_time"], ascending=[True])

                times = np.array(subdf.ABI_time)
                timediffs = (times[num_seq:] - times[0:-(num_seq)]).astype(
                    "timedelta64[m]"
                )

                # padding front end with zeros
                timediffs = np.concatenate((np.zeros(num_seq), timediffs.astype(int)))
                good_timeind = np.where(timediffs == deltaT)

                for gti in good_timeind[0]:
                    rec = []
                    rec.append(np.stack(subdf.ch13[gti - num_seq : gti + 1]))
                    for sv in scalar_vars:
                        rec.append(subdf.iloc[gti][sv])
                    # add to records if not empty
                    if rec:
                        (records.append(rec))

    newdf = pd.DataFrame.from_records(records)
    newdf.columns = grid_vars + scalar_vars

    newdf.to_pickle("ch13_seq_mayjune.pkl")
