import zarr
import numpy as np
import pandas as pd
import xarray as xr
from glob import glob
import os, sys
from datetime import datetime, timedelta
import time
from multiprocessing import Pool
from netCDF4 import Dataset

sys.path.insert(0, "..")

t0 = time.time()


def get_data(ncs, seq_len):

    recs = {
        "lat": [],
        "lon": [],
        "ABI_date": [],
        "ABI_time": [],
        "CH02": [],
        "CH05": [],
        "CH06": [],
        "CH13": [],
        "FED": [],
        "ABIx": [],
        "ABIy": [],
        "ABIx10": [],
        "ABIy10": [],
        "ABIx05": [],
        "ABIy05": [],
        "maxFED_t0": [],
        "maxFED_t5": [],
        "maxFED_t10": [],
        "maxFED_t15": [],
        "maxFED_t20": [],
        "maxFED_t25": [],
        "maxFED_t30": [],
        "SST": [],
        "mlcape": [],
        "u925": [],
        "v925": [],
        "u850": [],
        "v850": [],
        "u700": [],
        "v700": [],
        "u500": [],
        "v500": [],
        "ltg5to30": [],
    }
    atts = [
        "lat",
        "lon",
        "SST",
        "mlcape",
        "u925",
        "v925",
        "u850",
        "v850",
        "u700",
        "v700",
        "u500",
        "v500",
        "maxFED_t0",
        "maxFED_t5",
        "maxFED_t10",
        "maxFED_t15",
        "maxFED_t20",
        "maxFED_t25",
        "maxFED_t30",
        "ltg5to30",
        "ABI_date",
        "ABI_time",
        "ABIx",
        "ABIy",
        "ABIx10",
        "ABIy10",
        "ABIx05",
        "ABIy05",
    ]

    for s in ncs:
        variables = {"CH02": [], "CH05": [], "CH06": [], "CH13": [], "FED": []}

        print(s)
        # goes16_conus_Y112_X2182_20180611-001719.netcdf
        # find previous seq_len times
        basefile = os.path.basename(s)
        dirname = os.path.dirname(s)
        dirbase = os.path.dirname(dirname)  # remove yyyymmdd
        parts = basefile.split("_")
        dt = datetime.strptime(parts[-1][0:15], "%Y%m%d-%H%M%S")
        prev_dts = [dt - timedelta(minutes=sl * 5) for sl in range(seq_len)]
        prev_dts.reverse()

        cont = False
        for pp in range(len(prev_dts)):
            try:
                pnc = glob(
                    prev_dts[pp].strftime(
                        os.path.join(
                            dirbase,
                            "%Y%m%d",
                            ("_").join(
                                [
                                    parts[0],
                                    parts[1],
                                    parts[2],
                                    parts[3],
                                    "%Y%m%d-%H%M*.netcdf",
                                ]
                            ),
                        )
                    )
                )[0]
            except IndexError:
                cont = True
                break
            else:
                nc = Dataset(pnc, "r")
                nc.set_auto_mask(False)

                if pp == 0:  # test
                    # give tmp = 0 a 15% chance of going through
                    tmp = (
                        1
                        if (
                            np.max(
                                [
                                    nc.maxFED_t10,
                                    nc.maxFED_t15,
                                    nc.maxFED_t20,
                                    nc.maxFED_t25,
                                    nc.maxFED_t30,
                                ]
                            )
                            >= 1
                        )
                        else 0
                    )
                    if tmp == 0:
                        if np.random.random(1)[0] < 0.85:
                            cont = True
                            nc.close()
                            break
                if pp == (seq_len - 1):
                    # still need to add the last frame to vars here
                    for var in variables:
                        # QC if/elif blocks
                        if var in ["CH02", "CH05", "CH06"]:
                            if np.max(np.abs(nc.variables[var][:])) > 2:
                                cont = True
                                break
                        elif var == "CH13":
                            if np.max(np.abs(nc.variables[var][:])) > 500:
                                cont = True
                                break

                        variables[var].append(nc.variables[var][:])

                    # put atts after vars so that QCing of vars will short circuit this loop and we won't add atts to recs
                    if cont:
                        pass  # this just passes on to the next step...which should be closing the netcdf, then
                        # moving on to the next sample
                    else:
                        for att in atts:
                            if att == "ABI_date":
                                val = int(nc.ABI_time[0:8])
                            elif att == "ABI_time":
                                val = int(nc.ABI_time[9:13])
                            elif att == "ltg5to30":
                                val = (
                                    1
                                    if (
                                        np.max(
                                            [
                                                nc.maxFED_t10,
                                                nc.maxFED_t15,
                                                nc.maxFED_t20,
                                                nc.maxFED_t25,
                                                nc.maxFED_t30,
                                            ]
                                        )
                                        >= 1
                                    )
                                    else 0
                                )
                            else:
                                val = nc.getncattr(att)
                            recs[att].append(val)

                else:
                    for var in variables:
                        # QC if/elif blocks
                        if var in ["CH02", "CH05", "CH06"]:
                            if np.max(np.abs(nc.variables[var][:])) > 2:
                                cont = True
                                break
                        elif var == "CH13":
                            if np.max(np.abs(nc.variables[var][:])) > 500:
                                cont = True
                                break

                        variables[var].append(nc.variables[var][:])

                nc.close()

        if cont:
            continue  # insufficient number of previous frames

        for var in variables:
            variables[var] = np.stack(variables[var], axis=-1)
            recs[var].append(variables[var])

    return recs


##################################################################################################################

if __name__ == "__main__":
    date = sys.argv[1]

    nthreads = 15
    chunksize = 300  # number of images/samples to chunk or read at once
    seq_len = 1

    pool = Pool(nthreads)

    ncdir = "/data/jcintineo/AI/OCONUS/netcdf/"

    allncs = np.sort(glob(os.path.join(ncdir, date, "goes16_conus_*.netcdf*")))
    # perform shuffling here
    shuff_ind = np.random.permutation(len(allncs))
    allncs = allncs[shuff_ind]

    interval = int(np.ceil(len(allncs) / nthreads))

    list_of_lists = []

    for n in range(nthreads):
        list_of_lists.append(allncs[n * interval : n * interval + interval + 1])

    import itertools

    results = pool.starmap(get_data, zip(list_of_lists, itertools.repeat(seq_len)))
    pool.close()
    pool.join()

    from collections import OrderedDict

    fields = OrderedDict()
    fieldnames = [
        "lat",
        "lon",
        "ABI_date",
        "ABI_time",
        "CH02",
        "CH05",
        "CH06",
        "CH13",
        "FED",
        "maxFED_t0",
        "maxFED_t5",
        "maxFED_t10",
        "maxFED_t15",
        "maxFED_t20",
        "maxFED_t25",
        "maxFED_t30",
        "ltg5to30",
        "SST",
        "mlcape",
        "u925",
        "v925",
        "u850",
        "v850",
        "u700",
        "v700",
        "u500",
        "v500",
        "ABIx",
        "ABIy",
        "ABIx10",
        "ABIy10",
        "ABIx05",
        "ABIy05",
        "solzen",
    ]
    for f in fieldnames:
        fields[f] = []

    datasets = {
        "CH02": {
            "atts": {"desc": "0.65-micron refl from ABI", "units": "dimensionless"}
        },
        "CH05": {
            "atts": {"desc": "1.6-micron refl from ABI", "units": "dimensionless"}
        },
        "CH06": {
            "atts": {"desc": "2.26-micron refl from ABI", "units": "dimensionless"}
        },
        "CH13": {"atts": {"desc": "10.3-micron BT from ABI", "units": "K"}},
        "FED": {
            "atts": {
                "desc": "GLM flash extent density, accumulated over 5 min",
                "units": "fl",
            }
        },
        "lat": {"atts": {}},
        "lon": {"atts": {}},
        "ABI_date": {"atts": {}},
        "ABI_time": {"atts": {}},
        "SST": {"atts": {"desc": "source - NOAA OISST"}},
        "mlcape": {"atts": {"desc": "source - NOAA HRRR"}},
        "solzen": {"atts": {"desc": "solar zenith angle of center of image patch"}},
        "u925": {"atts": {"desc": "source - NOAA HRRR"}},
        "v925": {"atts": {"desc": "source - NOAA HRRR"}},
        "u850": {"atts": {"desc": "source - NOAA HRRR"}},
        "v850": {"atts": {"desc": "source - NOAA HRRR"}},
        "u700": {"atts": {"desc": "source - NOAA HRRR"}},
        "v700": {"atts": {"desc": "source - NOAA HRRR"}},
        "u500": {"atts": {"desc": "source - NOAA HRRR"}},
        "v500": {"atts": {"desc": "source - NOAA HRRR"}},
        "maxFED_t0": {"atts": {}},
        "maxFED_t5": {"atts": {}},
        "maxFED_t10": {"atts": {}},
        "maxFED_t15": {"atts": {}},
        "maxFED_t20": {"atts": {}},
        "maxFED_t25": {"atts": {}},
        "maxFED_t30": {"atts": {}},
        "ltg5to30": {
            "atts": {
                "desc": "GLM ltg present within pixel distance tolerance and within 5-30 min from ABI time"
            }
        },
        "ABIx": {"atts": {}},
        "ABIy": {"atts": {}},
        "ABIx10": {"atts": {}},
        "ABIy10": {"atts": {}},
        "ABIx05": {"atts": {}},
        "ABIy05": {"atts": {}},
    }

    # write zarr
    zname = "/apollo/grain/jcintineo/AI/OCONUS/pltg_8dates.zarr"
    ds_exists = True if (os.path.isdir(zname)) else False
    root = zarr.open(zname, mode="a")

    from lightningcast.solar_angles import solar_angles

    for f in fields:
        if f == "solzen":
            dts = [
                datetime.strptime(str(x) + str(y).zfill(4), "%Y%m%d%H%M")
                for x, y in zip(fields["ABI_date"], fields["ABI_time"])
            ]
            solzen, solaz = solar_angles(dt=dts, lon=fields["lon"], lat=fields["lat"])
            fields[f] = solzen
        else:
            for ii in range(
                len(results)
            ):  # concatenate list of results (dicts) and then stck them into an array
                fields[f] += results[ii][f]
            fields[f] = np.stack(fields[f])
            if f == "FED":
                ind = fields[f] > 9999
                fields[f][ind] = 0
        datasets[f]["data"] = fields[f]

        # set chunksizes
        if len(datasets[f]["data"].shape) == 1:
            chunks = (chunksize,)
        else:
            chunks = (
                chunksize,
                datasets[f]["data"].shape[1],
                datasets[f]["data"].shape[2],
                1,
            )

        # create data
        if ds_exists:  # just append along time axis
            newarr = zarr.zeros(
                datasets[f]["data"].shape,
                chunks=chunks,
                dtype=str(datasets[f]["data"].dtype),
            )
            newarr[:] = datasets[f]["data"]
            root[f].append(newarr, axis=0)
        else:  # start of new data array in group
            newarr = root.zeros(
                f,
                shape=datasets[f]["data"].shape,
                chunks=chunks,
                dtype=str(datasets[f]["data"].dtype),
            )
            newarr[:] = datasets[f]["data"]
            # set attributes
            for att in datasets[f]["atts"]:
                newarr.attrs[att] = datasets[f]["atts"][att]

    print(time.time() - t0)
