import netCDF4
import numpy as np
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import cartopy.feature as cfeature
import glob
import pyresample as pr
import sys

goes = 17
if goes == 16:
    # goes-east
    lats = np.arange(-6, 55.1, 0.1)
    lons = np.arange(-135, -29.9, 0.1)
    lon_0 = -75
else:
    # goes-west
    lats = np.arange(0, 65.1, 0.1)
    lons = np.arange(-175, -89.9, 0.1)
    lons, lats = np.meshgrid(lons, lats)
    lon_0 = -135

goespatt = f"/arcdata/goes/grb/goes{goes}/2021/2021_12_02_336/abi/L1b/RadF/*C13*1800*"
ncfile = glob.glob(goespatt)[0]

h = 35786023

nc = netCDF4.Dataset(ncfile, "r")
x = nc.variables["x"][::10] * h
y = nc.variables["y"][::10] * h
# Brazil domain
# x = nc.variables['x'][2700:4800] * h
# y = nc.variables['y'][2300:4500] * h
# rad = nc.variables['Rad'][2300:4500,2700:4800]
nc.close()

width = len(x)
height = len(y)
area_extent = (x.min(), y.min(), x.max(), y.max())
extent = [x.min(), x.max(), y.min(), y.max()]
# print(extent)
geoproj = ccrs.Geostationary(
    central_longitude=lon_0, sweep_axis="x", satellite_height=h
)

# for Brazil plot
# extent = [x.min(),x.max(),y.min(),y.max()]
# print(extent)
# fig = plt.figure(figsize=(10,10))
# ax=fig.add_axes([0,0,1,1],projection=geoproj)
##ax.set_extent(extent, crs=geoproj)
# ax.imshow(rad, transform=geoproj, extent=extent, cmap=plt.get_cmap('viridis'))
# ax.coastlines(color='black',linewidth=3.0)
# state_borders = cfeature.STATES.with_scale('50m')
# ax.add_feature(state_borders, edgecolor='black', linewidth=1.5)
# country_borders = cfeature.BORDERS.with_scale('50m')
# ax.add_feature(country_borders, edgecolor='black', linewidth=3)
# plt.show()
# sys.exit()

mask = np.ones((lons.shape))

oldgrid = pr.geometry.GridDefinition(lons=lons, lats=lats)
projection_dict = {"proj": "geos", "sweep": "x", "h": h, "lon_0": lon_0, "units": "m"}
targetArea = pr.geometry.AreaDefinition(
    "targetProj",
    "targetProj",
    "targetProj",
    projection_dict,
    width=width,
    height=height,
    area_extent=area_extent,
)
mask = pr.kd_tree.resample_nearest(
    oldgrid, mask, targetArea, radius_of_influence=16000, fill_value=-1
)

fig = plt.figure(figsize=(10, 10))
ax = fig.add_axes([0, 0, 1, 1], projection=geoproj)
# ax.set_extent(extent, crs=geoproj)
ax.imshow(
    mask,
    origin="upper",
    transform=geoproj,
    extent=extent,
    vmin=0,
    vmax=2,
    cmap=plt.get_cmap("viridis"),
)
ax.coastlines(color="black", linewidth=2.0)
plt.show()
