# Grab a config object
import inspect, os
from lightningcast.utils import init_config_env, extract_keypath_values
import logging
from logging.config import dictConfig
import lightningcast.logging_def as ld
import argparse


def new_logger(logfile):
    os.makedirs(os.path.dirname(logfile), exist_ok=True)
    # Set up logging
    logDict = ld.logging_def(logfile=logfile)
    dictConfig(logDict)
    # send stderr to logger
    stderr_logger = logging.getLogger("STDERR")
    sl = ld.StreamToLogger(stderr_logger, logging.ERROR)


def run_tests(config):

    for test, subdict in config["std_tests"].items():  # could also do for key, value

        if test == "model":
            model = config[
                "std_tests.model"
            ]  # config['std_tests']['model'].replace('{lc_base}',config['lc_base'])
        elif test == "model_config_file":
            model_config_file = config[
                "std_tests.model_config_file"
            ]  # config['std_tests']['model_config_file'].replace('{lc_base}',config['lc_base'])
        else:

            # Run tests
            try:
                suffix = subdict["suffix"]
            except KeyError:
                suffix = None
            try:
                ind_bbox = subdict["ind_bbox"]
            except KeyError:
                ind_bbox = []
            try:
                ll_bbox = subdict["ll_bbox"]
            except KeyError:
                ll_bbox = []
            try:
                plax_hgt = subdict["plax_hgt"]
            except KeyError:
                plax_hgt = 0

            new_logger(subdict["logfile"])

            predict_ltg(
                subdict["filelist"],
                model,
                model_config_file,
                subdict["sector"],
                outdir=subdict["outdir"],
                datadir_patt=subdict["datadir_patt"],
                sector_suffix=suffix,
                ind_bbox=ind_bbox,
                ll_bbox=ll_bbox,
                savegrid=subdict["savegrid"],
                make_json=subdict["make_json"],
                grplacefile=subdict["grplacefile"],
                plax_hgt=plax_hgt,
                sat_info=config["sat_info"],
            )  # sat_info from nccf.yaml


parser = argparse.ArgumentParser(
    description="Runs a set of standard tests for given LightningCast sectors \
                      and creates various outputs such as netCDFs, jsons, and placefiles. See config/standard_tests.yaml for test details."
)

parser.add_argument(
    "inroot",
    help="Full path of directory containing filelists, and satellite subdirs. E.g., /data/PLTG/LightningCast_testdata/ ",
    type=str,
)
parser.add_argument("outroot", help="Full path of output directory.", type=str)

args = parser.parse_args()

lc_base = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
format_context = {
    "lc_base": lc_base,
    "pwd": os.getcwd(),
    "inroot": args.inroot,
    "outroot": args.outroot,
}

config_yamls = [
    os.path.join(root, file)
    for root, dirs, files in os.walk(os.path.abspath(f"{lc_base}/config/default/"))
    for file in files
]
config_yamls.sort()
# Add NCCF yaml
config_yamls.append(f"{lc_base}/config/nccf.yaml")
# Add standard_tests.yaml
config_yamls.append(f"{lc_base}/config/standard_tests.yaml")
yaml_overrides = {}


config = init_config_env(config_yamls, yaml_overrides, format_context)

from lightningcast.predict_ltg import (
    predict_ltg,
)  # This needs to be after env variables are set. When we switch to doing config differently it can maybe be moved to the top.

run_tests(config)
