import os, sys

# os.environ['CUDA_DEVICE_ORDER']="PCI_BUS_ID"
# os.environ["CUDA_VISIBLE_DEVICES"]="-1"
import tensorflow as tf
from tensorflow.keras.models import load_model
import tensorflow_addons as tfa
import pickle
import numpy as np
from lightningcast import losses
import copy
from lightningcast import utils
from lightningcast import ams_utils

# ------------------------------------------------------------------------------------------------------------------------------------------------------
def get_metrics():

    csi05 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.05,
        name="csi05",
    )
    csi10 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.1,
        name="csi10",
    )
    csi15 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.15,
        name="csi15",
    )
    csi20 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.2,
        name="csi20",
    )
    csi25 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.25,
        name="csi25",
    )
    csi30 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.3,
        name="csi30",
    )
    csi35 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.35,
        name="csi35",
    )
    csi40 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.4,
        name="csi40",
    )
    csi45 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.45,
        name="csi45",
    )
    csi50 = losses.csi(
        use_as_loss_function=False,
        use_soft_discretization=False,
        hard_discretization_threshold=0.5,
        name="csi50",
    )

    CUSTOM_METRICS = {
        "aupr": tf.keras.metrics.AUC(name="aupr", curve="PR"),
        "brier_score": tf.keras.metrics.MeanSquaredError(name="brier_score"),
        "csi05": csi05,
        "csi10": csi10,
        "csi15": csi15,
        "csi20": csi20,
        "csi25": csi25,
        "csi30": csi30,
        "csi35": csi35,
        "csi40": csi40,
        "csi45": csi45,
        "csi50": csi50,
    }
    METRICS = [
        "accuracy",
        tf.keras.metrics.AUC(name="aupr", curve="PR"),
        tf.keras.metrics.MeanSquaredError(name="brier_score"),
        csi05,
        csi10,
        csi15,
        csi20,
        csi25,
        csi30,
        csi35,
        csi40,
        csi45,
        csi50,
    ]

    return CUSTOM_METRICS, METRICS


# ------------------------------------------------------------------------------------------------------------------------------------------------------

# Define a function to parse a single tfrecord example
def parse_tfrecord_fn(example):

    feature_description = {
        "CH02": tf.io.FixedLenFeature([], tf.string),
        "CH05": tf.io.FixedLenFeature([], tf.string),
        "CH08": tf.io.FixedLenFeature([], tf.string),
        "CH09": tf.io.FixedLenFeature([], tf.string),
        "CH10": tf.io.FixedLenFeature([], tf.string),
        "CH11": tf.io.FixedLenFeature([], tf.string),
        "CH13": tf.io.FixedLenFeature([], tf.string),
        "CH15": tf.io.FixedLenFeature([], tf.string),
        "FED_accum_60min_2km": tf.io.FixedLenFeature([], tf.string),
    }
    features = tf.io.parse_single_example(example, feature_description)
    features["input_0"] = tf.reshape(
        tf.io.parse_tensor(features["CH02"], tf.uint8), [1920, 1920, 1]
    )
    features["input_1"] = tf.reshape(
        tf.io.parse_tensor(features["CH05"], tf.uint8), [960, 960, 1]
    )
    ch08 = tf.reshape(tf.io.parse_tensor(features["CH08"], tf.uint8), [480, 480, 1])
    ch09 = tf.reshape(tf.io.parse_tensor(features["CH09"], tf.uint8), [480, 480, 1])
    ch10 = tf.reshape(tf.io.parse_tensor(features["CH10"], tf.uint8), [480, 480, 1])
    ch11 = tf.reshape(tf.io.parse_tensor(features["CH11"], tf.uint8), [480, 480, 1])
    ch13 = tf.reshape(tf.io.parse_tensor(features["CH13"], tf.uint8), [480, 480, 1])
    ch15 = tf.reshape(tf.io.parse_tensor(features["CH15"], tf.uint8), [480, 480, 1])
    fed_accum = tf.reshape(
        tf.io.parse_tensor(features["FED_accum_60min_2km"], tf.uint8), [480, 480, 1]
    )

    features["input_2"] = tf.concat([ch08, ch09, ch10, ch11, ch13, ch15], axis=-1)

    # binarize
    zero = tf.zeros_like(fed_accum)
    one = tf.ones_like(fed_accum)
    targets = tf.where(fed_accum >= 1, x=one, y=zero)

    return (features, targets)


# ------------------------------------------------------------------------------------------------------------------------------------------------------
# Define a function to permute a specific channel (feature) within all samples
def permute_channel(features, targets, input_name_and_chan_idx):
    input_name, chan_idx = input_name_and_chan_idx
    tensor = features[input_name]
    nsamples, ny, nx, nchans = tensor.shape

    if (
        nchans == 1
    ):  # If only one channel in the input, shuffle that channel and replace in features
        features[input_name] = tf.expand_dims(
            tf.random.shuffle(tensor[:, :, :, chan_idx], seed=803), axis=-1
        )
    else:  # If > 1 channel in the input, make a new tensor with only the selected channel shuffled.
        tmp = []
        for ii in range(nchans):
            if ii == chan_idx:
                tmp.append(
                    tf.expand_dims(
                        tf.random.shuffle(tensor[:, :, :, chan_idx], seed=803), axis=-1
                    )
                )
            else:
                tmp.append(tf.expand_dims(tensor[:, :, :, ii], axis=-1))

        features[input_name] = tf.concat(tmp, axis=-1)

    return (features, targets)


# ------------------------------------------------------------------------------------------------------------------------------------------------------

AUTOTUNE = tf.data.AUTOTUNE

# Specify the number of GPUs
NGPU = 4

# Load the model (with metric(s))
model_file = (
    "/ships22/grain/probsevere/LC/tests/2023/c0205080910111315/fit_conv_model.h5"
)

# Load config file
config_file = os.path.dirname(model_file) + "/model_config.pkl"
config = pickle.load(open(config_file, "rb"))
inputs = config["inputs"]

# Create the test dataset for running the permutation
sat = "goes18"
# val_list = [f"/ships22/grain/probsevere/LC/tfrecs3/{sat}/2023/202309??/*-1[3-9]*tfrec",
#            f"/ships22/grain/probsevere/LC/tfrecs3/{sat}/2023/202309??/*-2[0-2]*tfrec"]
val_list = [
    f"/ships22/grain/probsevere/LC/tfrecs3/{sat}/2023/2023??2[5-9]/*-1[7-9]*tfrec",
    f"/ships22/grain/probsevere/LC/tfrecs3/{sat}/2023/2023??2[5-9]/*-2*tfrec",
    f"/ships22/grain/probsevere/LC/tfrecs3/{sat}/2023/2023??3?/*-1[7-9]*tfrec",
    f"/ships22/grain/probsevere/LC/tfrecs3/{sat}/2023/2023??3?/*-2*tfrec",
]
test_filenames = tf.io.gfile.glob(val_list)
n_samples = len(test_filenames)
print("\nNumber of samples:", n_samples, "\n")
test_ds = (
    tf.data.TFRecordDataset(test_filenames, num_parallel_reads=AUTOTUNE)
    .map(parse_tfrecord_fn, num_parallel_calls=AUTOTUNE)
    .batch(64)  # Don't batch if making numpy arrays
    .prefetch(AUTOTUNE)
)


if NGPU > 1:
    # Create a MirroredStrategy.
    strategy = tf.distribute.MirroredStrategy()
    print("Number of devices: {}".format(strategy.num_replicas_in_sync))
    with strategy.scope():
        custom_objs, metrics = get_metrics()
        conv_model = load_model(model_file, custom_objects=custom_objs)
else:
    custom_objs, metrics = get_metrics()
    conv_model = load_model(model_file, custom_objects=custom_objs)

# Disable AutoShard.
options = tf.data.Options()
options.experimental_distribute.auto_shard_policy = (
    tf.data.experimental.AutoShardPolicy.OFF
)
test_ds = test_ds.with_options(options)


# Get original cost (before permutation).
eval_results = conv_model.evaluate(test_ds)  # for tfrecorddataset
original_cost = eval_results[conv_model.metrics_names.index("aupr")]
print("Original cost (no permutation): {0:.4e}\n".format(original_cost))

# Create all_chans_dict, which gives input and channel index for each channel
all_chans_dict = {}
# Also create remaining_pred_names (a list of all channels, to begin with).
remaining_pred_names = []
for ii, inp in enumerate(config["inputs"]):
    for jj, chan in enumerate(inp):
        all_chans_dict[chan] = [f"input_{ii}", jj]  # if ds
        # all_chans_dict[chan] = [ii,jj] #the input and channel index
        remaining_pred_names.append(chan)

current_step_num = 0
permanent_permute_chans = []  # list of channels that have been permuted
permuted_predictor_name_by_step = []
highest_cost_by_step = []
predictor_names_step1 = []
costs_step1 = []

while len(remaining_pred_names) > 0:

    current_step_num += 1

    highest_cost = np.inf
    best_predictor_name = None
    best_predictor_permuted_values = None

    for this_pred_name in remaining_pred_names:

        print(
            f"Trying predictor {this_pred_name} at step {current_step_num} of permutation test..."
        )

        curr_permute_chans = copy.deepcopy(permanent_permute_chans)
        curr_permute_chans.append(
            this_pred_name
        )  # Becomes a list of channels to permute

        # Permute channels sequentially, starting with the pure test_ds
        for ii, curr_permute_chan in enumerate(curr_permute_chans):
            print(f"Shuffling {curr_permute_chan}")
            # Note that x is a dict of inputs and y is a Tensor of targets
            if ii == 0:
                permuted_ds = test_ds.map(
                    lambda x, y: permute_channel(
                        x, y, all_chans_dict[curr_permute_chan]
                    ),
                    num_parallel_calls=AUTOTUNE,
                )
            else:
                permuted_ds = permuted_ds.map(
                    lambda x, y: permute_channel(
                        x, y, all_chans_dict[curr_permute_chan]
                    ),
                    num_parallel_calls=AUTOTUNE,
                )

        eval_results = conv_model.evaluate(permuted_ds)
        # eval_results = conv_model.evaluate(permuted_inputs,y=targets) #for numpy
        this_cost = eval_results[conv_model.metrics_names.index("aupr")]

        if current_step_num == 1:
            predictor_names_step1.append(this_pred_name)
            costs_step1.append(this_cost)

        if (
            this_cost > highest_cost
        ):  # i.e., if this is less negative (not as much cost), then continue
            continue

        highest_cost = this_cost + 0.0
        best_predictor_name = this_pred_name + ""

    permuted_predictor_name_by_step.append(best_predictor_name)
    highest_cost_by_step.append(highest_cost)

    # Remove best predictor from list.
    remaining_pred_names.remove(best_predictor_name)

    # Leave values of best predictors permuted.
    # A list of predictors to permute on future steps.
    permanent_permute_chans.append(best_predictor_name)

    print(
        f'\nBest predictor = "{best_predictor_name}" ... new cost = {np.round(highest_cost,4)}\n'
    )


# Write out results
result_dict = {
    "permuted_predictor_name_by_step": permuted_predictor_name_by_step,
    "highest_cost_by_step": np.array(highest_cost_by_step),
    "original_cost": original_cost,
    "predictor_names_step1": predictor_names_step1,
    "costs_step1": np.array(costs_step1),
}

outdir = os.path.dirname(model_file) + "/permutation_val_2/"
output_pickle_file_name = f"{outdir}/permutation_results.pkl"
utils.mkdir_p(outdir)

print(f'Writing results to: "{output_pickle_file_name}"...')
pickle.dump(result_dict, open(output_pickle_file_name, "wb"))

# Create images for breiman (one-pass)and successive permutation tests.
breiman_file_name = f"{outdir}/breiman.png"
ams_utils.plot_breiman_results(
    result_dict=result_dict,
    output_file_name=breiman_file_name,
    plot_percent_increase=False,
)

lakshmanan_file_name = f"{outdir}/successive.png"
ams_utils.plot_lakshmanan_results(
    result_dict=result_dict,
    output_file_name=lakshmanan_file_name,
    plot_percent_increase=False,
)

print(result_dict)
